/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s): Damien Fournier
 * 
 */

package org.ow2.frascati.explorer.action;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.filechooser.FileFilter;

import org.mortbay.log.Log;
import org.objectweb.fractal.api.Component;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.ow2.frascati.explorer.ExplorerGUI;
import org.ow2.frascati.explorer.context.AssemblyFactoryContext;

import de.schlichtherle.io.swing.JFileChooser;

/**
 * This class provides the ability to load SCA composites
 * via the Load Menu item {@link MenuItem}.
 *
 * @author Christophe Demarey, Philippe Merle
 */

public class LoadMenuItem
  implements MenuItem
{
  // --------------------------------------------------------------------------
  // Internal state
  // --------------------------------------------------------------------------

  /** The {@link JFileChooser} instance. */
  protected static JFileChooser fileChooser = null;

  // --------------------------------------------------------------------------
  // Internal methods
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Constructor
  // --------------------------------------------------------------------------

  /**
   * The default constructor.
   */
  public LoadMenuItem () {
  }

  // --------------------------------------------------------------------------
  // Implementation of the MenuItem interface
  // --------------------------------------------------------------------------

  /*
   * @see org.objectweb.util.explorer.api.MenuItem#getStatus(org.objectweb.util.explorer.api.TreeView)
   */
  public int getStatus (final TreeView treeView) {
    return MenuItem.ENABLED_STATUS;
  }

  /*
   * @see org.objectweb.util.explorer.api.MenuItem#actionPerformed(org.objectweb.util.explorer.api.MenuItemTreeView)
   */
  public void actionPerformed (final MenuItemTreeView e) {
    // At the first call, creates the file chooser.
    if(fileChooser == null) {
      fileChooser = new JFileChooser();
      // Sets to the user's current directory.
      fileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
      // Filters SCA files.
      fileChooser.setFileFilter(            
        new FileFilter() {
          /** Whether the given file is accepted by this filter. */
          public boolean accept(File f) {
            String extension = f.getName().substring(f.getName().lastIndexOf('.') + 1);
            return extension.equalsIgnoreCase("composite") || f.isDirectory();
          }
          /** The description of this filter. */
          public String getDescription() {
            return "SCA composite files";
          }
        }
      );
    }

    // Opens the file chooser to select a file.
    int returnVal = fileChooser.showOpenDialog(null);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File f = fileChooser.getSelectedFile();
      String name = f.toString(); 
      int index = name.indexOf(".jar");
      
      if ( index != -1 ) { // File in a jar
        // Add the jar to the classpath
        URL[] urls = new URL[1];
        try {
          urls[0] = new File(name.substring(0, index+4)).toURI().toURL();
        } catch (MalformedURLException e1) {
          Log.warn(e1.getMessage());
        }
        
        ExplorerGUI explorer =  ExplorerGUI.getSingleton();
        explorer.getFactoryLoadItf().loadJars(urls);
        
        // Load the composite
        loadComposite( name.substring(index + 5) );
      } else {
        loadComposite( f.getPath() );
      }
    }
  }
  
  // --------------------------------------------------------------------------
  // Public methods.
  // --------------------------------------------------------------------------

  /**
   * Load a SCA composite file.
   * 
   * @param compositeName - The name of the composite file.
   */
  public void loadComposite(String compositeName) {
    Component c = null;
    ExplorerGUI explorer = ExplorerGUI.getSingleton();
    String baseDir = System.getProperty("java.io.tmpdir");
    
    explorer.getFactoryJuliacItf().setOutputDir( new File(baseDir) );
    //    try {
      c = (Component) explorer.getFactoryLoadItf().loadComposite(compositeName);
//    } catch (java.lang.ClassNotFoundException cnfe) {
//      System.err.println("===========> ERROR!!!!");
//    } catch (Exception e) {
//      System.err.println("===========> ERROR!!!!");      
//    }
      
    if (c != null)
      // ExplorerGUI.getSingleton().addComponent(c);
      AssemblyFactoryContext.addEntry(c);
    else
      Log.warn("Cannot load "+compositeName+"!");
  }
  
}