/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */
package org.ow2.frascati.explorer.context;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.explorer.FcExplorer;
import org.objectweb.fractal.explorer.context.ComponentContextForService;
import org.objectweb.fractal.explorer.context.InternalComponentContainer;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.ContextContainer;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.ow2.frascati.tinfi.control.intent.SCAIntentController;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;

/**
 * Component context for user role.
 *  
 * @author Christophe Demarey.
 */
public class ComponentContext
     extends ComponentContextForService 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    private static final long serialVersionUID = 6420654128175193919L;
    
    /** This Context Container contains the controller interfaces. */
    protected ContextContainer controllers = null;
    
    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tells if this component owns SCA properties or not.
     * 
     * @param pc The SCA Property Controller of this component.
     * @return True if this component has at least one property.
     */
    private boolean hasProperties(SCAPropertyController pc) {
      return !pc.getAll().isEmpty();
    }
    
    /**
     * Tells if this component has SCA intents or not.
     * 
     * @param ic The SCA Intent Controller of this component.
     * @return True if this component has at least one intent.
     */
    private boolean hasIntents(SCAIntentController ic) {
      // TODO Not yet implemented
      return false;
    }
    
    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================
    
    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public Entry[] getEntries(Object object) {
      Component ci = (Component) object;
      List<Entry> additionalEntries = new ArrayList<Entry>();
      
      // ** For SCA composites
      try {
          ContentController cc = FcExplorer.getContentController(ci);
          ContextContainer componentsContainer = new InternalComponentContainer();
          Component[] components = cc.getFcSubComponents();
          
          for (Component c : components) {
            try {
              c.getFcInterface(org.ow2.frascati.tinfi.control.component.ComponentContextDef.NAME);
              componentsContainer.addEntry(FcExplorer.getName(c), c);
            } catch (NoSuchInterfaceException e) {
              /* The sca-component-controller cannot be found!
                 This component is not a SCA component => Nothing to do! */
            }
          }
          
          additionalEntries.add( new DefaultEntry("components", componentsContainer) );
      } catch (NoSuchInterfaceException e) {
          // Nothing to do
      }
      
      // ** Display SCA Properties
      try {
        SCAPropertyController propCtl = (SCAPropertyController) ci.getFcInterface(SCAPropertyController.NAME);

        if ( hasProperties(propCtl) )
          additionalEntries.add( new DefaultEntry("properties", propCtl) );
      } catch (NoSuchInterfaceException e) {
        e.printStackTrace();
      }
      
      // ** Display SCA intents
      /*
      try {
        SCAIntentController intentCtl = (SCAIntentController) ci.getFcInterface(SCAIntentController.NAME);

        if ( hasIntents(intentCtl) )
          additionalEntries.add( new DefaultEntry("intents", intentCtl) );
      } catch (NoSuchInterfaceException e) {
          e.printStackTrace();
      }*/
      
      // ** Return entries
      int nbAdditionalEntries = additionalEntries.size();
      Entry[] defaultEntries = super.getEntries(object);
      Entry[] entries = new Entry[defaultEntries.length + nbAdditionalEntries];

      // Copy additional entries
      short index = 0;
      for(Entry e : additionalEntries)
          entries[index++] = e;
      // Copy default entries
      System.arraycopy(defaultEntries, 0, entries, nbAdditionalEntries, defaultEntries.length);
      return entries;
    }
}
