/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */

package org.ow2.frascati.explorer.scaproperties;

import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import org.objectweb.fractal.explorer.attributes.AttributeValueBooleanCellEditor;
import org.objectweb.fractal.explorer.attributes.AttributeValueCellRenderer;
import org.objectweb.fractal.explorer.attributes.AttributesTableColumnModel;
import org.objectweb.fractal.explorer.panel.MonitorCellRenderer;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;

/**
 * An extended <code>JTable</code> that represents the properties managed
 * by the component.
 * 
 * @author Christophe Demarey.
 * @author Jakub Kornas.
 */
public class PropertiesTable
     extends JTable {
    /** The SCAPropertyController "hosting" properties */
    protected SCAPropertyController propertiesController = null;

    private TableCellEditor monitorEditor = null;

    private TableCellRenderer monitorRenderer = null;

    private TableCellEditor propertyValueDefaultEditor = null;

    private TableCellEditor propertyValueBooleanEditor = null;

    private TableCellRenderer propertyValueRenderer = null;

    /**
     * Creates a new attributes table.
     * 
     * @param selectedNode
     *            a SCA property controller.
     * @param owner
     *            the <code>JPanel</code> to which this table belongs.
     */
    public PropertiesTable(SCAPropertyController selectedNode, JPanel owner) {
        this.propertiesController = selectedNode;

        this.setColumnModel(new AttributesTableColumnModel());
        this.setModel(PropertiesTableModelBuilder
                .buildAttributesTableModel(propertiesController));

        dataModel.addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                int row = e.getFirstRow();
                int column = e.getColumn();
                if (column == 1) {
                    if (getValueAt(row, 2).equals(
                            PropertiesTableModel.MONITOR_ACTIONS[1])) {
                        return;
                    }

                    String propertyName = getValueAt(row, 0).toString();
                    Object newValue = dataModel.getValueAt(row, column);
                    boolean invoke = false;
                    Class invokeType = null;
                    
                    if (newValue instanceof String) {
                        invokeType = String.class;
                        invoke = true;
                    } else if (newValue instanceof Integer) {
                        invokeType = int.class;
                        invoke = true;
                    } else if (newValue instanceof Boolean) {
                        invokeType = boolean.class;
                        invoke = true;
                    } else if (newValue instanceof Short) {
                        invokeType = short.class;
                        invoke = true;
                    } else if (newValue instanceof Long) {
                        invokeType = long.class;
                        invoke = true;
                    } else if (newValue instanceof Double) {
                        invokeType = double.class;
                        invoke = true;
                    } else if (newValue instanceof Float) {
                        invokeType = float.class;
                        invoke = true;
                    }
                    if (invoke) {
                        try {
                            propertiesController.set(propertyName, newValue);
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                }
            }
        });

        this.monitorEditor = new MonitorCellEditor(new JCheckBox());
        this.monitorRenderer = new MonitorCellRenderer();

        this.propertyValueDefaultEditor = new PropertyValueDefaultCellEditor( new JTextField() );
        this.propertyValueBooleanEditor = new AttributeValueBooleanCellEditor( new JCheckBox() );
        this.propertyValueRenderer = new AttributeValueCellRenderer();

        this.getColumn("PropertyValue").setCellRenderer(this.propertyValueRenderer);
        this.getColumn("Monitor").setCellRenderer(this.monitorRenderer);
    }

    /**
     * @see JTable#getDragEnabled()
     */
    public boolean getDragEnabled() {
        return false;
    }

    /**
     * @see JTable#getCellEditor(int, int)
     */
    public TableCellEditor getCellEditor(int row, int col) {
        if (col == 1) {
            Object value = getValueAt(row, col);
            if (value instanceof Boolean)
                return propertyValueBooleanEditor;
            return propertyValueDefaultEditor;
        } else if (col == 2) {
            return monitorEditor;
        } else {
            return super.getCellEditor(row, col);
        }
    }

    /**
     * @see JTable#getToolTipText(MouseEvent)
     */
    public String getToolTipText(MouseEvent e) {
        String tip = null;
        Point p = e.getPoint();
        int rowIndex = rowAtPoint(p);
        int colIndex = columnAtPoint(p);
        int realColumnIndex = convertColumnIndexToModel(colIndex);

        if (realColumnIndex == 1) {
            TableModel model = getModel();
            String propertyName = (String) model.getValueAt(rowIndex, 0);
            if (isCellEditable(rowIndex, colIndex)) {
                Object propertyValue = model.getValueAt(rowIndex, 1);
                if (propertyValue instanceof Boolean) {
                    boolean boolValue = ((Boolean) propertyValue)
                            .booleanValue();
                    tip = "Click to set the '" + propertyName
                            + "' property to '" + !boolValue + "'";
                } else {
                    tip = "Click to modify the '" + propertyName
                            + "' property";
                }
            } else {
                tip = "Property '" + propertyName + "' is non-modifiable";
            }
        } else if (realColumnIndex == 2) {
            TableModel model = getModel();
            String propertyName = (String) model.getValueAt(rowIndex, 0);

            String option = model.getValueAt(rowIndex, 2).toString();
            tip = "Click to " + option + " monitoring the '" + propertyName
                    + "' property";
        }
        return tip;
    }

    /**
     * Destroys all attribute monitoring threads created through user's
     * interaction with this attributes table.
     */
    public void destroyPropertyMonitors() {
        PropertiesTableModel aModel = (PropertiesTableModel) dataModel;

        PropertyDescriptor[] descriptors = aModel.getPropertyDescriptors();
        for (PropertyDescriptor desc : descriptors) {
            if (desc != null && desc.hasMonitor()) {
                PropertyMonitoringThread monitor = (PropertyMonitoringThread) desc.getMonitor();
                monitor.running = false;
            }
            desc = null;
        }
    }

    /**
     * Returns the property controller associated to this properties table.
     * 
     * @return the property controller associated to this table.
     */
    public SCAPropertyController getPropertyController() {
        return this.propertiesController;
    }
}

