/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.dispatcher;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import org.eclipse.stp.sca.Composite;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.assembly.ScaAssembly;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.Sca2Ecore;

/**
 * Manage Assembly Factory instantiation process
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class ManagerImpl implements Manager {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ManagerImpl.class.getCanonicalName());

  /**
   * The required model component
   */

  @Requires(name = "model")
  public Sca2Ecore model;

  /**
   * The required assembly builder component
   */

  @Requires(name = "assembly")
  public ScaAssembly assembly;

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  public RuntimeFactory runtime;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#loadComposite(java.lang
   * .String)
   */

  public Object loadComposite(String composite) {

    // SCA Composite instance given by EMF
    Composite compositeModel = null;

    // Component Object return by factory
    Object componentObject = null;

    // Load composite as URI using internal Loader
    URI compositeFileURI = runtime.getCompositeResource(composite);

    // Call the EMF parser component
    log.fine("Reading composite " + composite);

    try {
      // Use SCA parser to create composite model instance
      compositeModel = model.parse(compositeFileURI);
    } catch (IOException e) {
      log.severe("Error when loading the composite file " + composite
          + " with the SCA XML Processor");
      e.printStackTrace();
      System.exit(1);
    }

    // Return the resulting assembly object (fractal composite)
    try {
      componentObject = assembly.create(compositeModel);
    } catch (Exception e) {
      e.printStackTrace();
    }

    return componentObject;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.dispatcher.Manager#loadJars(java.net.URL[])
   */

  public ClassLoader loadJars(URL[] urls) {
    
    // Get list of URL for logging
    try {
      for (URL url : urls) {
        File test = new File(url.toURI());
        // Test if URL exist
        if (!test.exists())
          log.warning("File does not exist : " + url.toString());
        else
          log.info("Load library : " + url.toString());
      }
    } catch (URISyntaxException e) {
      e.printStackTrace();
    }
    
    runtime.loadLibraries(urls);
    
    return runtime.getClassLoader();
  }

  public void finalize() {
    this.model = null;
    this.assembly = null;
  }

}
