/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.util.HashMap;

import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.WebServiceBinding;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.bf.BindingFactory;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.util.Fractal;

/**
 * Bind components using Web Service Binding
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaBindingWs implements ScaBindingProtocol {
  
  /**
   * The binding type according to the SCA model 
   */
  
  public static int BINDING_TYPE = ScaPackage.WEB_SERVICE_BINDING;

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaBindingWs.class.getCanonicalName());

  @Requires(name = "binding-factory")
  BindingFactory bf;

  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#bind(org.eclipse.stp.sca.BaseReference, org.eclipse.stp.sca.Binding, org.objectweb.fractal.api.Component)
   */
  
  public void bind(BaseReference reference, Binding binding, Component component) {

    WebServiceBinding b = (WebServiceBinding) binding;
    String bindingURI = b.getUri();
    String referenceName = ScaBindingUtil.getReferenceName(reference);

    HashMap<String, String> m = new HashMap<String, String>();
    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.ADDRESS,
        bindingURI);
    m.put(ScaBindingProtocol.PLUGIN_ID, "ws");

//    if (reference.eClass().getClassifierID() == ScaPackage.REFERENCE) {
//      String[] promoted = ScaBindingUtil.getPromote(((Reference) reference));
//      component = FractalUtil.getFractalSubComponent(promoted[0], component);
//    }

    try {
      log.fine("Calling binding factory\n bind : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + referenceName + "\n with "
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.ADDRESS
          + "=" + b.getUri());

      bf.bind(component, referenceName, m);

    } catch (BindingFactoryException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }

  }


  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#export(org.eclipse.stp.sca.BaseService, org.eclipse.stp.sca.Binding, org.objectweb.fractal.api.Component)
   */
  
  public void export(BaseService service, Binding binding, Component component) {

    WebServiceBinding b = (WebServiceBinding) binding;
    HashMap<String, String> m = new HashMap<String, String>();
    String serviceName = ScaBindingUtil.getServiceName(service);

    m.put(org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.ADDRESS,
        b.getUri());
    m.put(ScaBindingProtocol.PLUGIN_ID, "ws");
    
//    if (service.eClass().getClassifierID() == ScaPackage.SERVICE) {
//      String[] promoted = ScaBindingUtil.getPromote(((Service) service));
//      component = FractalUtil.getFractalSubComponent(promoted[0], component);
//    }

    try {

      log.fine("Calling binding factory\n export : "
          + Fractal.getNameController(component).getFcName() + " -> "
          + serviceName + "\n with "
          + org.objectweb.fractal.bf.connectors.ws.WsConnectorConstants.ADDRESS
          + "=" + b.getUri());

      bf.export(component, serviceName, m);

    } catch (BindingFactoryException e) {
      e.printStackTrace();
    } catch (NoSuchInterfaceException e) {
      log.severe("Error while getting component name controller");
      e.printStackTrace();
    }
  }

  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.binding.ScaBindingProtocol#getBindingID()
   */
  
  public int getBindingID() {
    return BINDING_TYPE;
  }

}
