/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.implementation;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.stp.sca.domainmodel.frascati.FractalImplementation;
import org.eclipse.stp.sca.Implementation;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;

/**
 * Create SCA component implemented with Fractal
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaFractalImpl implements ScaImplementation {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaFractalImpl.class.getCanonicalName());
  
  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  private RuntimeFactory runtime;


  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#create(
   * org.eclipse.stp.sca.Implementation,
   * org.objectweb.fractal.api.type.ComponentType)
   */

  public Component create(Implementation implementation,
      ComponentType componentType) throws ImplementationException {

    Component c = null;
    FractalImplementation impl = (FractalImplementation) implementation;

    try {
      // Try loading from definition generated with Juliac
      Class<?> classe = runtime.getClassLoader().loadClass(impl.getDescription());
      org.objectweb.fractal.api.factory.Factory object = (org.objectweb.fractal.api.factory.Factory) classe
          .newInstance();

      // Create component instance
      c = object.newFcInstance();
      return c;

    } catch (ClassNotFoundException e) {

      // Generated class not found try with fractal definition and Fractal ADL
      org.objectweb.fractal.adl.Factory f;

      try {
        // init Fractal ADL factory
        Map<Object, Object> context = new HashMap<Object, Object>();
        context.put("classloader", runtime.getClassLoader());
        f = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND);

        // try instanciation for the given Fractal ADL definition
        c = (Component) f.newComponent(impl.getDescription(), context);
        return c;

      } catch (ADLException ex) {
        // Error with Fractal ADL
        log.severe("Error in the Fractal implementation : "
            + impl.getDescription());
        ex.printStackTrace();
      }
    } catch (InstantiationException e) {
      throw new ImplementationException(
          "Error when building instance for class " + impl.getDescription(), e);
    } catch (IllegalAccessException e) {
      throw new ImplementationException("Could not access class "
          + impl.getDescription(), e);
    } catch (org.objectweb.fractal.api.factory.InstantiationException e) {
      throw new ImplementationException(
          "Error when building component instance : " + impl.getDescription(), e);
    }

    return c;

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.instance.implementation.ScaImplementation#
   * getImplementationID()
   */

  public int getImplementationID() {
    return org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage.FRACTAL_IMPLEMENTATION;
  }

}
