/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */
package org.ow2.frascati.factory.core.instance.property;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.stp.sca.Property;
import org.eclipse.stp.sca.PropertyValue;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.tinfi.control.property.IllegalPromoterException;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.sun.xml.bind.api.impl.NameConverter;

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaPropertyImpl implements ScaProperty {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaPropertyImpl.class.getCanonicalName());

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.property.ScaProperty#setProperty(org.eclipse
   * .stp.sca.Property, org.objectweb.fractal.api.Component)
   */

  public void setProperty(Property property, Component component) {

    try {

      SCAPropertyController pc = (SCAPropertyController) component
          .getFcInterface("sca-property-controller");

      QName propertyObject = (property.getElement() != null) ? property
          .getElement() : property.getType();

      Map<Object, Object> options = new HashMap<Object, Object>();

      options.put(org.eclipse.emf.ecore.xmi.XMLResource.OPTION_ROOT_OBJECTS,
          Collections.singletonList(property));

      Document document = ((XMLResource) property.eResource()).save(null,
          options, null);

      NodeList nList = document.getChildNodes().item(0).getChildNodes();

      Node n = nList.item(0);

      Boolean textNode = true;

      while (n != null) {

        switch (n.getNodeType()) {

        case Node.ELEMENT_NODE:

          textNode = false;
          String packagename = NameConverter.standard
              .toPackageName(propertyObject.getNamespaceURI());
          log.finer("Resolve complex property");
          log.finer("Property package name is : " + packagename);

          JAXBContext jc = JAXBContext.newInstance(packagename);
          Unmarshaller u = jc.createUnmarshaller();

          org.w3c.dom.Element element = (org.w3c.dom.Element) n;
          Object o = u.unmarshal(element);

          pc.set(property.getName(), o);

          break;

        case Node.TEXT_NODE:

          if (!textNode)
            break;

          log.finer("Resolve simple text property");

          org.w3c.dom.Text text = (org.w3c.dom.Text) n;
          String prop = text.getTextContent();
          if (prop != null) // if prop is null implies empty xml node
            pc.set(property.getName(), prop);
          break;
        }

        n = n.getNextSibling();

      }

    } catch (NoSuchInterfaceException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (JAXBException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.property.ScaProperty#setProperty(org.eclipse
   * .stp.sca.PropertyValue, org.objectweb.fractal.api.Component)
   */

  public void setProperty(PropertyValue property, Component component) {
    try {

      SCAPropertyController pc = (SCAPropertyController) component
          .getFcInterface("sca-property-controller");

      String source = property.getSource();
      if ((source != null) && (!source.equals(""))) {
        if (source.startsWith("$"))
          source = source.substring(1);

        Component superComponents[] = Fractal.getSuperController(component)
            .getFcSuperComponents();        
        
        SCAPropertyController superPc = (SCAPropertyController) superComponents[0]
            .getFcInterface("sca-property-controller");

        try {
          pc.setPromoter(source, superPc);
        } catch (IllegalPromoterException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }

      }

      else {

        QName propertyObject = (property.getElement() != null) ? property
            .getElement() : property.getType();

        Map<Object, Object> options = new HashMap<Object, Object>();

        options.put(org.eclipse.emf.ecore.xmi.XMLResource.OPTION_ROOT_OBJECTS,
            Collections.singletonList(property));

        Document document = ((XMLResource) property.eResource()).save(null,
            options, null);

        NodeList nList = document.getChildNodes().item(0).getChildNodes();

        Node n = nList.item(0);

        Boolean textNode = true;

        while (n != null) {

          switch (n.getNodeType()) {

          case Node.ELEMENT_NODE:

            textNode = false;
            String packagename = NameConverter.standard
                .toPackageName(propertyObject.getNamespaceURI());
            log.finer("Resolve complex property");
            log.finer("Property package name is : " + packagename);

            JAXBContext jc = JAXBContext.newInstance(packagename);
            Unmarshaller u = jc.createUnmarshaller();

            org.w3c.dom.Element element = (org.w3c.dom.Element) n;
            Object o = u.unmarshal(element);

            pc.set(property.getName(), o);

            break;

          case Node.TEXT_NODE:

            if (!textNode)
              break;

            log.finer("Resolve simple text property");

            org.w3c.dom.Text text = (org.w3c.dom.Text) n;
            String prop = text.getTextContent();
            if (prop != null) // if prop is null implies empty xml node
              pc.set(property.getName(), prop);
            break;
          }

          n = n.getNextSibling();

        }
      }

    } catch (NoSuchInterfaceException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (JAXBException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } 
  }

}
