/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.parser;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.DocumentRoot;
import org.eclipse.stp.sca.ScaPackage;
import org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage;
import org.eclipse.stp.sca.domainmodel.tuscany.TuscanyPackage;
import org.eclipse.stp.sca.util.ScaResourceFactoryImpl;
import org.objectweb.fractal.fraclet.annotations.Component;

/**
 * Provide Ecore model for this ADL
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class Sca2EcoreImpl implements Sca2Ecore {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.frascati.factory.core.parser.Sca2Ecore#parse(java.lang.String)
	 */

	public Composite parse(URI uri) throws IOException {
		
	  // Create a resource set to hold the resources.
		ResourceSet resourceSet = new ResourceSetImpl();

		// Register the appropriate resource factory to handle all file
		// extensions.
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("*", new ScaResourceFactoryImpl());

		// Register the EMF packages
		resourceSet.getPackageRegistry().put(ScaPackage.eNS_URI,
				ScaPackage.eINSTANCE);
		
		resourceSet.getPackageRegistry().put(FrascatiPackage.eNS_URI,
				FrascatiPackage.eINSTANCE);
		
		resourceSet.getPackageRegistry().put(TuscanyPackage.eNS_URI,
				TuscanyPackage.eINSTANCE);
		
		org.eclipse.emf.common.util.URI resourceUri = org.eclipse.emf.common.util.URI.createURI(uri.toString());
		
		// Parse the composite file indicated by its uri
		Resource resource = resourceSet.getResource(resourceUri, true);

		// Get Root node of the composite model
		EList<EObject> list = resource.getContents();
		Composite composite = ((DocumentRoot) list.get(0)).getComposite();

		// Save memory
		list = null;
		resource = null;

		return composite;
	}

}
