/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.parser.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.Reference;
import org.eclipse.stp.sca.Service;

public class PromoteResolver implements Resolver {

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.parser.util.Resolver#resolve(org.eclipse.
   * stp.sca.Composite)
   */

  /**
   * Resolve promote attribute
   */

  public Composite resolve(Composite composite) {

    // Create collection of reference to resolve promote attribute
    ArrayList<Reference> references = new ArrayList<Reference>();
    // Create collection of service to resolve promote attribute
    ArrayList<Service> services = new ArrayList<Service>();

    // collection of components
    HashMap<String, Component> components = new HashMap<String, Component>();

    EList<Component> list = composite.getComponent();

    for (Component component : list) {
      // register components by name
      components.put(component.getName(), component);
    }

    // register incomplete promote reference
    for (Reference reference : composite.getReference()) {
      String promote = reference.getPromote();
      if ((promote != null) && (promote.split("/").length < 2)) {
        references.add(reference);
      }
    }

    // register incomplete promote service
    for (Service service : composite.getService()) {
      String promote = service.getPromote();
      if ((promote != null) && (promote.split("/").length < 2)) {
        services.add(service);
      }
    }

    // set missing promote reference attribute
    for (Reference reference : references) {
      Component c = components.get(reference.getPromote());
      EList<ComponentReference> cReferences = c.getReference();
      Iterator<ComponentReference> i = cReferences.iterator();

      // if composite reference does not have interface then
      // promote the first reference of the promoted component
      if (reference.getInterface() == null)
        reference.setPromote(reference.getPromote() + "/"
            + cReferences.get(0).getName());

      // else promote the first reference which provides same interface type
      else {

        boolean found = false;
        while (i.hasNext() && !found) {
          ComponentReference componentReference = i.next();
          // Compare Interfaces using a comparator class
          InterfaceComparator comparator = new InterfaceComparator();
          if (comparator.compare(componentReference.getInterface(), reference
              .getInterface()) == 0) {
            // Same interfaces, can add the reference name to the target
            // attribute
            String oldname = reference.getPromote();
            String newname = oldname + "/" + componentReference.getName();
            reference.setPromote(newname);
            found = true;
          }
        }
      }

    }

    // set missing promote service attribute
    for (Service service : services) {
      Component c = components.get(service.getPromote());
      EList<ComponentService> cServices = c.getService();
      Iterator<ComponentService> i = cServices.iterator();

      // if composite service does not have interface then
      // promote the first service of the promoted component
      if (service.getInterface() == null)
        service.setPromote(service.getPromote() + "/"
            + cServices.get(0).getName());

      // else promote the first service which provides same interface type
      else {

        boolean found = false;
        while (i.hasNext() && !found) {
          ComponentService componentService = i.next();
          // Compare Interfaces using a comparator class
          InterfaceComparator comparator = new InterfaceComparator();
          if (comparator.compare(componentService.getInterface(), service
              .getInterface()) == 0) {
            // Same interfaces, can add the reference name to the target
            // attribute
            String oldname = service.getPromote();
            String newname = oldname + "/" + componentService.getName();
            service.setPromote(newname);
            found = true;
          }
        }
      }

    }

    return composite;

  }
}
