/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.parser.util;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;

public class TargetResolver implements Resolver {

  /*
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.core.parser.util.Resolver#resolve(org.eclipse.stp.sca.Composite)
   */

  /**
   * Resolve Reference target attribute
   */
  
  public Composite resolve(Composite composite) {

    // Create collection of reference to resolve target
    ArrayList<ComponentReference> references = new ArrayList<ComponentReference>();

    // collection of components
    HashMap<String, Component> components = new HashMap<String, Component>();

    EList<Component> list = composite.getComponent();

    for (Component component : list) {
      // register components by name
      components.put(component.getName(), component);
      EList<ComponentReference> cRefList = component.getReference();

      // register incomplete target references
      for (ComponentReference componentReference : cRefList) {
        String target = componentReference.getTarget();
        if ((target != null) && (target.split("/").length < 2)) {
          references.add(componentReference);
        }
      }
    }

    // set missing targeted service name
    for (ComponentReference reference : references) {
      Component c = components.get(reference.getTarget());
      EList<ComponentService> services = c.getService();
      for (ComponentService componentService : services) {
        // Compare Interfaces using a comparator class
        InterfaceComparator comparator = new InterfaceComparator();
        if (comparator.compare(componentService.getInterface(), reference
            .getInterface()) == 0) {
          // Same interfaces, can add the reference name to the target attribute
          String oldname = reference.getTarget();
          String newname = oldname + "/" + componentService.getName();
          reference.setTarget(newname);
        }
      }

    }

    return composite;
  }
}
