/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.generate.component;

import java.util.ArrayList;
import java.util.Iterator;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.ow2.frascati.factory.core.instance.NotSupportedException;
import org.ow2.frascati.factory.core.instance.binding.BindingException;
import org.ow2.frascati.factory.core.instance.component.ScaComponent;
import org.ow2.frascati.factory.core.instance.implementation.ImplementationException;
import org.ow2.frascati.factory.core.instance.implementation.ScaImplementation;
import org.ow2.frascati.factory.core.instance.interfaces.InterfaceException;
import org.ow2.frascati.factory.core.instance.interfaces.ScaInterface;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;

/**
 * Create components instances
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class ScaComponentGenerateImpl implements ScaComponent {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger("assembly-factory");

  /**
   * The required runtime
   */

  @Requires(name = "runtime")
  RuntimeFactory runtime;

  /**
   * The implementation switcher component
   */

  @Requires(name = "implementation")
  ScaImplementation implementation;

  /**
   * The java interface builder component
   */

  @Requires(name = "component-interface")
  ScaInterface itf;

  public org.objectweb.fractal.api.Component create(
      org.eclipse.stp.sca.Component comp) throws ImplementationException, NotSupportedException, BindingException, InterfaceException {

    log.fine("Generating component " + comp.getName());

    ComponentType componentType;
    componentType = create_componentType(comp);
    log.fine("Generate component " + comp.getName() + " instance");
    implementation.create(comp.getImplementation(), componentType);
    return null;

  }

  // TODO Update according to the assembly factory component architecture
  public org.objectweb.fractal.api.Component create(
      org.eclipse.stp.sca.Composite composite) throws ImplementationException, NotSupportedException, BindingException, InterfaceException {

    @SuppressWarnings("unused")
    org.objectweb.fractal.api.Component assemblyComponent = null;
    ComponentType assemblyType = null;

    try {
      // Get the composite definition
      log.info("Building composite " + composite.getName());
      
      // Create container
      ComponentType containerType = runtime.createFcType(new InterfaceType[0]);
      org.objectweb.fractal.api.Component container = runtime.newFcInstance(
          containerType, "scaComposite", null);

      // Create assembly
      assemblyType = create_componentType(composite);
      assemblyComponent = runtime
          .newFcInstance(assemblyType, "scaComposite", null);

      // Generate assembly components and add them to the assembly
      EList<org.eclipse.stp.sca.Component> components = composite
          .getComponent();
      for (org.eclipse.stp.sca.Component c : components) {
        create(c);
      }

      log.info("Composite " + composite.getName()
          + " successfully generated \n");

    } catch (InstantiationException e) {
      e.printStackTrace();
    }

    return null;
  }

  /**
   * Create Fractal component type from the component definition in the model
   * 
   * @param comp
   *          - Component definition according to the model
   * @return - Resulting Component type
   */

  public ComponentType create_componentType(EObject comp) throws InterfaceException {

    try {

      // Create a new Collection of fractal interfaces type
      ArrayList<InterfaceType> interfaceType = new ArrayList<InterfaceType>();

      // Get list of services for this component/composite Object
      EList<?> slist = (EList<?>) comp.getClass().getMethod("getService")
          .invoke(comp);
      Iterator<?> si = slist.iterator();

      // Get list of refrences for this component/composite Object
      EList<?> rlist = (EList<?>) comp.getClass().getMethod("getReference")
          .invoke(comp);
      Iterator<?> ri = rlist.iterator();

      // Creates interface type for services
      while (si.hasNext()) {
        BaseService cs = (BaseService) si.next();
        try {
          interfaceType.add(itf.create(cs.getInterface()));
        } catch (InterfaceException e) {
          
          //Retrieve SCA Component 
          EObject c = cs.eContainer();
          String name = (String) c.getClass().getMethod("getName")
          .invoke(c);
          
          throw new InterfaceException(
              "Could not create interface type for " + name + " with service " +  cs.getName(), e);
        }
      }

      // Creates interface type for references
      while (ri.hasNext()) {
        BaseReference cr = (BaseReference) ri.next();
        try {
          interfaceType.add(itf.create(cr.getInterface()));
        } catch (InterfaceException e) {
          
          //Retrieve SCA Component 
          EObject c = cr.eContainer();
          String name = (String) c.getClass().getMethod("getName")
          .invoke(c);
          throw new InterfaceException(
              "Could not create interface type for " + name + " with reference " + cr.getName(),
              e);
        }
      }

      InterfaceType[] itftype = interfaceType
          .toArray(new InterfaceType[interfaceType.size()]);

      log.finer("Create interface type for component");

      ComponentType componentType = runtime.createFcType(itftype);
      if (componentType == null)
        log.severe("Returned component type is NULL");

      return componentType;

    } catch (Exception e) {
      String name = null;
      try {
        name = (String) comp.getClass().getMethod("getName").invoke(comp);
      } catch (Exception e1) {
        // Nothing more to do
      }
      
      throw new InterfaceException("Could not create interfaces type for component " + name,
          e);

    } 

  }

}
