/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.annotation.Annotation;

import org.ow2.frascati.tinfi.TinfiException;

/**
 * Exception thrown if a reference is defined more than once.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class DuplicateAnnotatedElementException extends TinfiException {
    
    private static final long serialVersionUID = 5458263663005313791L;

    private String refname;
    private Class<?> cl;
    private Class<? extends Annotation> annot;
    
    public DuplicateAnnotatedElementException(
        String refname, Class<?> cl, Class<? extends Annotation> annot ) {
        
        super();
        this.refname = refname;
        this.cl = cl;
        this.annot = annot;
    }
    
    @Override
    public String getMessage() {
        String msg =
            "Several fields and/or methods annotated with @"+
            annot.getSimpleName()+" for reference "+refname+" in class "+
            cl.getName();
        return msg;
    }
}
