/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.ow2.frascati.tinfi.control.content.SCAContentController;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;

/**
 * Mixin layer to be includes in the lifecycle controller for managing the
 * EagerInit policy.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public abstract class LifeCycleEagerInitMixin implements LifeCycleController {
    
    // -------------------------------------------------------------------------
    // Private constructor
    // -------------------------------------------------------------------------

    private LifeCycleEagerInitMixin () {}

    // -------------------------------------------------------------------------
    // Implementation of the LifeCycleController interface
    // -------------------------------------------------------------------------

    public void startFc() throws IllegalLifeCycleException {
        _super_startFc();
        
        /*
         * Notify the SCA content controller that the component has been started.
         * This gives the opportunity to eager initialize the component if the
         * @EagerInit annotation has been set.
         */
        try {
            _this_weavableSCACC.eagerInit();
        }
        catch( InstantiationException ie ) {
            throw new ChainedIllegalLifeCycleException(ie,null,"");
        }
    }
    
    public void stopFc() throws IllegalLifeCycleException {
        _super_stopFc();
    }
        

    // -------------------------------------------------------------------------
    // Fields and methods required by the mixin class in the base class
    // -------------------------------------------------------------------------

    public abstract void _super_startFc() throws IllegalLifeCycleException;
    public abstract void _super_stopFc() throws IllegalLifeCycleException;

    /**
     * The {@link SCAPropertyController} interface of the component to which
     * this controller object belongs.
     */
    public SCAContentController _this_weavableSCACC;
}
