/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.reflect.Method;

/**
 * Utility methods.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class Util {

    /**
     * Check whether the specified method is a setter for the specified
     * property.
     * 
     * @param method    the method to be checked
     * @param propname  the property name (may be null)
     * @param proptype  the property type (may be null)
     * @throws IllegalArgumentException
     *      if the method is not a setter for the specified property
     */
    public static void checkSetterMethodSignature(
        Method method, String propname, Class<?> proptype )
    throws IllegalArgumentException {
    
        /*
         * First, check that this is void:() method with a name which starts
         * with set.
         */
        if( ! method.getName().startsWith("set") ) {
            String msg =
                "The name of the setter method "+method.getName()+
                " should start with set";
            throw new IllegalArgumentException(msg);
        }
        if( method.getParameterTypes().length != 1 ) {
            String msg =
                "Only one parameter should be defined for setter method "+
                method.getName();
            throw new IllegalArgumentException(msg);
        }
        if( ! method.getReturnType().equals(void.class) ) {
            String msg =
                "Return type for setter method "+method.getName()+
                " should be void";
            throw new IllegalArgumentException(msg);
        }
        
        /*
         * Check the property name.
         */
        if( propname != null ) {
            // fooBar -> setFooBar
            String methodName = "set"+propname.substring(0,1).toUpperCase()+propname.substring(1);            
            if( ! method.getName().equals(methodName) ) {
                String msg =
                    "Method "+method.getName()+
                    " is not a setter method for property name "+propname;
                throw new IllegalArgumentException(msg);
            }
        }
        
        /*
         * Check the property type.
         */
        if( proptype != null ) {
            Class<?> paramcl = method.getParameterTypes()[0];
            if( ! proptype.isAssignableFrom(paramcl) ) {
                String msg =
                    "Method "+method.getName()+
                    " is not a setter method for property type "+proptype;
                throw new IllegalArgumentException(msg);
            }
        }
    }
}
