/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.metadata;

import java.lang.reflect.Method;

/**
 * Utility methods.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 0.4.4
 */
public class Util {

    /**
     * Return <code>true</code> if the specified method is a setter method.
     * 
     * @param setter  a setter method
     * @return  <code>true</code> if <code>setter</code> is a setter method
     */
    public static boolean isSetterMethod( Method setter ) {
        
        String name = setter.getName();
        if( ! name.startsWith("set") ) {
            return false;
        }
        Class<?> rtype = setter.getReturnType();
        if( ! rtype.equals(void.class) ) {
            return false;
        }
        Class<?>[] ptypes = setter.getParameterTypes();
        if( ptypes.length != 1 ) {
            return false;
        }
        
        return true;
    }

    /**
     * Return the name of the property set by the specified setter method.
     * 
     * @param setter  a setter method
     * @return  the name of the property set by <code>setter</code>
     * @throws IllegalArgumentException
     *      if <code>setter</code> is not a setter method
     */
    public static String getPropertyName( Method setter ) {
        
        if( ! isSetterMethod(setter) ) {
            String msg = setter.toString() + " is not a setter method";
            throw new IllegalArgumentException(msg);
        }
        
        String name = setter.getName();
        name = name.substring(3);   // Skip set
        name = name.substring(0,1).toLowerCase() + name.substring(1);
        
        return name;
    }
}
