/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.osoa;

import org.osoa.sca.CallableReference;
import org.osoa.sca.annotations.Conversational;

/**
 * Implementation of a callable reference.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class CallableReferenceImpl<B> implements CallableReference<B> {
    
    public CallableReferenceImpl( Class<B> businessInterface, B service ) {
        this.businessInterface = businessInterface;
        this.service = service;        
        this.conversationAnnotation =
            businessInterface.getAnnotation(Conversational.class);
    }
    
    // ----------------------------------------------------------------------
    // Implementation of the ServiceReference interface
    // ----------------------------------------------------------------------
    
    private Class<B> businessInterface;
    private B service;

    public Class<B> getBusinessInterface() {
        return businessInterface;
    }

    public Object getCallbackID() {
        return callbackID;
    }

    /**
     * Returns the current conversation associated with this reference.
     *
     * @return the current conversation associated with this reference; null if there is no active conversation
     * @throws IllegalStateException if this reference is not conversational
     */
    public ConversationImpl getConversation() throws IllegalStateException {
        if( ! isConversational() ) {
            String msg = "This reference is not conversational";
            throw new IllegalStateException(msg);
        }
        if( conversation == null ) {
            conversation = new ThreadLocal<ConversationImpl>();
        }
        ConversationImpl conv = conversation.get();
        if( conv == null ) {
            conv = new ConversationImpl();
            conversation.set(conv);
        }
        return conv;
    }

    public B getService() {
        return service;
    }

    public boolean isConversational() {
        return (conversationAnnotation != null);
    }

    private Conversational conversationAnnotation;
    protected Object callbackID;
    
    /** The conversation associated with the current reference. */
    private ThreadLocal<ConversationImpl> conversation;
}
